#ifndef _rfsv16_h_
#define _rfsv16_h_

#include "rfsv.h"

class rfsv16 : public rfsv {
	public:
	rfsv16(ppsocket *skt);
	~rfsv16();
	void reset(); // these 2 added
	void reconnect();

	// these are the original publics
	long dir(const char* dirName, bufferArray *files);
	/*long read(const char* psionName, const char* localName);
	long write(const char* localName, const char* psionName);*/
	long mkdir(const char* dirName);

	// these are FS' promotions
	long dircount(const char *name, long *count);
	long copyFromPsion(const char *from, const char *to);
	long copyToPsion(const char *from, const char *to);
	long rmdir(const char *name);
	long remove(const char *name); // MJG: was this del?
	long rename(const char *oldname, const char *newname);
	long mktemp(long *handle, char *tmpname);
	long fgeteattr(const char *name, long *attr, long *size, long *time);
	long fgetattr(const char *name, long *attr);
	long fsetattr(const char *name, long seta, long unseta);
	long fgetmtime(const char *name, long *mtime);
	long fsetmtime(const char *name, long mtime);
	long fopendir(long attr, const char *name, long &handle);
	long fopen(long attr, const char *name, long &handle);
	long fcreatefile(long attr, const char *name, long &handle);
	long freplacefile(long attr, const char *name, long &handle);
	long fseek(long handle, long pos, long mode);
	long fread(long handle, char *buf, long len);
	long fwrite(long handle, char *buf, long len);
	long fsetsize(long handle, long size);
	long fclose(long handle);

	long devlist(long *devbits);
	char *devinfo(int devnum, long *vfree, long *vtotal, long *vattr, long *vuiqueid);
	long getStatus();
	char *opErr(long status);
	char *opAttr(long status);

	enum errs {
		NO_ERROR = 0,
		E_FILE_EXIST = -32,
		E_FILE_NXIST = -33,
		E_FILE_WRITE = -34,
		E_FILE_READ = -35,
		E_FILE_EOF = -36,
		E_FILE_FULL = -37,
		E_FILE_NAME = -38,
		E_FILE_ACCESS = -39,
		E_FILE_LOCKED = -40,
		E_FILE_DEVICE = -41,
		E_FILE_DIR = -42,
		E_FILE_RECORD = -43,
		E_FILE_RDONLY = -44,
		E_FILE_INV = -45,
		E_FILE_PENDING = -46,
		E_FILE_VOLUME = -47,
		E_FILE_CANCEL = -48,
		E_FILE_ALLOC = -10,
		E_FILE_DISC = -50,
		E_FILE_CONNECT = -51,
		E_FILE_RETRAN = -52,
		E_FILE_LINE = -53,
		E_FILE_INACT = -54,
		E_FILE_PARITY = -55,
		E_FILE_FRAME = -56,
		E_FILE_OVERRUN = -57,
		E_MDM_CONFAIL = -58,
		E_MDM_BUSY = -59,
		E_MDM_NOANS = -60,
		E_MDM_BLACKLIST = -61,
		E_FILE_NOTREADY = -62,
		E_FILE_UNKNOWN = -63,
		E_FILE_DIRFULL = -64,
		E_FILE_PROTECT = -65,
		E_FILE_CORRUPT = -66,
		E_FILE_ABORT = -67,
		E_FILE_ERASE = -68,
		E_FILE_INVALID = -69,
	};

 
private:
  enum commands {
    FOPEN = 0, // File Open
    FCLOSE = 2, // File Close
    FREAD = 4, // File Read
    FDIRREAD = 6, // Read Directory entries
    FDEVICEREAD = 8, // Device Information
    FWRITE = 10, // File Write
    FSEEK = 12, // File Seek
    FFLUSH = 14, // Flush
    FSETEOF = 16,
    RENAME = 18,
    DELETE = 20,
    FINFO = 22,
    SFSTAT = 24,
    PARSE = 26,
    MKDIR = 28,
    OPENUNIQUE = 30,
    STATUSDEVICE = 32,
    PATHTEST = 34,
    STATUSSYSTEM = 36,
    CHANGEDIR = 38,
    SFDATE = 40,
    RESPONSE = 42
  };
  
  enum fopen_attrib {
    P_FOPEN = 0x0000, /* Open file */
    P_FCREATE = 0x0001, /* Create file */
    P_FREPLACE = 0x0002, /* Replace file */
    P_FAPPEND = 0x0003, /* Append records */
    P_FUNIQUE = 0x0004, /* Unique file open */
    P_FSTREAM = 0x0000, /* Stream access to a binary file */
    P_FSTREAM_TEXT = 0x0010, /* Stream access to a text file */
    P_FTEXT = 0x0020, /* Record access to a text file */
    P_FDIR = 0x0030, /* Record access to a directory file */
    P_FFORMAT = 0x0040, /* Format a device */
    P_FDEVICE = 0x0050, /* Record access to device name list */
    P_FNODE = 0x0060, /* Record access to node name list */
    P_FUPDATE = 0x0100, /* Read and write access */
    P_FRANDOM = 0x0200, /* Random access */
    P_FSHARE = 0x0400 /* File can be shared */
  };

  enum status_enum {
    P_FAWRITE = 0x0001, /* can the file be written to? */    
    P_FAHIDDEN = 0x0002, /* set if file is hidden */    
    P_FASYSTEM = 0x0004, /* set if file is a system file */
    P_FAVOLUME = 0x0008, /* set if the name is a volume name */    
    P_FADIR = 0x0010, /* set if file is a directory file */    
    P_FAMOD = 0x0020, /* has the file been modified? */    
    P_FAREAD = 0x0100, /* can the file be read? */    
    P_FAEXEC = 0x0200, /* is the file executable? */
    P_FASTREAM = 0x0400, /* is the file a byte stream file? */    
    P_FATEXT = 0x0800 /* is it a text file? */
  };
  
  const char *getConnectName();

  // File handlers
  //long fopen(fopen_attrib a, const char* file, int &handle); // returns status 0=OK
  //long fclose(int fileHandle);

	// Miscellaneous
	int convertName(const char* orig, char *retVal);

	// Communication
	bool sendCommand(enum commands c, bufferStore &data);
	long getResponse(bufferStore &data);
  
	// Vars
	ppsocket *skt;
	// MJG: not sure what these are yet
	int serNum;
	long status;	// current connection status
	int tDiff;	// don't think this is used anywhere
};

#endif
